/**************************************************************************
******************* RESOLUCION DE OBJETIVOS. ****************************
***************************************************************************
* FICHERO ob.c: Se implementa la resolucion de objetivos mediante dos metodos.
* El Metodo Clasico, empleando unificacion semantica por narrowing, y el
* Metodo Semantico empleando unificacion sintactica contra la semantica
* operacional O(E), o contra la semantica de punto fijo, F(E), ya que ambas
* coinciden. Los unicos puntos de entrada de este fichero son:
*
*	resob_cla(); Resolucion de objetivos mediante el Metodo Clasico.
*
*	resob_sem(); Resolucion de objetivos mediante el Metodo Semantico.
*
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "const.h"
#include "estruc.h"
#include "glovar.h"

/* Prototipos de funciones externas, definidas en Procesos auxiliares.
No todas estas funciones son empleadas en "ob.c". */ 
#include "proaux.h"

/* Prototipos de funciones externas, definidas en Fichero de Unificacion
Sintactica. No todas estas funciones son empleadas en "ob.c". */
#include "unific.h"

/* Prototipos de funciones externas, definidas en Fichero de Unificacion
Semantica. No todas estas funciones son empleadas en "ob.c". */
#include "narrow.h"


/******   RESOLUCION DE OBJETIVOS MEDIANTE EL METODO CLASICO  ********/

/* Toma como argumento la lista de estados del objetivo y genera el estado de 
narrowing inicial, que devuelve como resultado. El campo raiz no se necesita 
para nada, ya que el objetivo a tratar es unico. */

PRIVATE struct stat *genstat(struct lista *obj)
{
	struct stat *estaux=NULL;
	if (obj!=NULL) {
	  if ((estaux= (struct stat *) malloc(sizeof(struct stat))) == NULL)
					memerror();
	  estaux->ecus=copilis(obj); /* lista ecs. del objetivo */
	  estaux->raiz=NULL; /* No se necesita */
	  estaux->alfa=NULL; /* Substitucion indentidad, E */
	  estaux->sig=NULL;
	}
	return estaux;
}


/* Visualizacion de objetivos resueltos. Por la forma en que calcula las 
respuestas computadas el metodo de Basic Narrowing, basta con aplicar al 
objetivo inicial cada una de las sustituciones que contienen los estados 
de exito. No altera ninguna de las estructuras con las que trabaja. */

PRIVATE verres(struct stat *exitos)
{
	int i; /* para estetica */
	struct arbol *e;
	struct lista *l; /* recorre la lista de ecuaciones del objetivo */
	struct stat *s; /* recorre la lista de estados de exito */
	if (exitos)
	{
	   printf("\nRESULTADOS: INSTANCIACION DEL OBJETIVO.");
	   for (s=exitos, i=0; s; s=s->sig, i++) {
		/* visualiza la substitucion a aplicar */
		printf("\n\nSUBSTITUCION theta(%i) = ", i);
		verunif(s->alfa); printf("\n");
		/* visualiza la instancia del objetivo */
		for (l=goalini; l; l=l->sig) {
			e=copiecu(l->ecu);
			aplicar(&e, s->alfa);
			verecu(e);
			if (l != goalfin) printf(" , ");
			else printf("\n");
			borraecu(e);
		}
	   }
	}
	else printf("\nEL OBJETIVO ES IRRESOLUBLE.");
}


/******************************************************************************
Resolucion de un objetivo mediante el metodo clasico. Los pasos a seguir son
bien sencillos:

A)	Se genera un estado de narrowing inicial.
B)	Se aplica basic narrowing a dicho estado, para obtener las respuestas 
	computadas.
C)	Se visualizan las instancias del objetivo de acuerdo a las respuestas. 
*******************************************************************************/

PUBLIC resob_cla()
{
	struct stat *S, /* puntero al estado de narrowing inicial */
		    *E; /* puntero a la lista de estados de exito.
			   Contienen las repuestas computadas. */
	start_st(CLASICO);
	S = genstat(goalini);
	E = narrower(S); borrastat(S);
	verres(E); borrastat(E);
	stop_st();
}



/**************************************************************************
******   RESOLUCION DE OBJETIVOS MEDIANTE EL METODO SEMANTICO   *******
**************************************************************************/

/****************   GESTION DE LA MEMORIA    ******************************/

/*elimina toda la memoria asociada a una sustitucion, pero
teniendo cuidado de no borrar la lista de ecuaciones
apuntada por el campo ecus o raiz. Esta es otro detalle para
propiciar la eficiencia del proceso de resolucion de objetivos.
Nos evitariamos realizar un monton de copias, referenciando las
ecuaciones, mediante punteros a la lista de ecuaciones del estado
objetivo inicial.
Lo de NE es para indicar que No borra las Ecuaciones. */

PRIVATE borrastatNE (struct stat *est)
{ if (est) {
	borrastatNE(est->sig);
	borrasub(est->alfa);
	free(est);};
}


/*************** DIVISION DE LA LISTA DE ECUACIONES PLANAS ********************
*******************************************************************************
*
* Este procedimiento realiza la separacion de la lista de ecuaciones en dos 
* listas: una lista de ecuaciones flat no triviales, de la forma f(X1,..., Xn)
* =Xn+1; y una lista de ecuaciones flat triviales de la forma Xk=Xk+1.
* Referencias a estas listas estaran almacenadas en las variables ntgoalini y 
* tgoalini, respectivamente.
*
*******************************************************************************/

PRIVATE struct lista *ntgoalini, *tgoalini;

/* Separa la lista en dos. Se limita a recorrer la lista de ecuaciones buscando
ecuaciones del tipo Xk=Xk+1. Altera la lista de ecuaciones flat, fgoalini, que 
queda deshecha. */

PRIVATE split(struct lista *flatobj)
{
	struct lista *l, *ntfin=NULL, *tfin=NULL;
	struct arbol *izq, *der;
	
	ntgoalini=NULL; tgoalini=NULL;	
	for (l=flatobj; l; l=l->sig) {
		izq=l->ecu; der=l->ecu->herm;
		if (esvar(izq) && esvar(der)) {
			if (tgoalini==NULL) tgoalini=l;
			else tfin->sig=l;
			tfin=l;
		}
		else {
			if (ntgoalini==NULL) ntgoalini=l;
			else ntfin->sig=l;
			ntfin=l;
		}
	};
	if (tfin!=NULL) tfin->sig=NULL; 
	if (ntfin!=NULL) ntfin->sig=NULL;
	if (trace) {
		verlis("\nLISTA FLAT TRIVIAL: ", tgoalini);
		verlis("\nLISTA FLAT  NO TRIVIAL: ", ntgoalini);
	};
}

/**************  UNIFICACION SINTACTICA CONTRA LA TEORIA  **************
***************************************************************************
*
* Lo que llamamos Metodo Semantico, va a consistir en comprobar la unificacion
* sintactica de las ecuaciones del objetivo contra cada una de las ecuaciones
* estandarizadas de la Semantica Operacional, O(E).
*
* Esencialmente, esto se realiza mediante la aplicacion reiterada de la
* siguiente regla de unificacion, RU:
*
*		g=Fsel(G) && e<<O(E) && sigma=mgu(g.theta, e)
* 	----------------------------------------------------------
*		<<=G, theta> -RU-> <<=(G-{g}), theta.sigma>
*
* donde los <<=G, theta> son ESTADOS DE RESOLUCION, formados por un objetivo y 
* una sustitucion. Por simplicidad, seguiremos utilizando la estructura stat 
* para representar estos estados.
*
* Mas formalmente una RESOLUCION DE UN OBJETIVO contra O(E), sera una secuencia
* de transiciones de estado:
*
*	<<=G0, ident> -RU-> <<=G1, theta(1)> -RU-> ... -RU-> <<=Gn, theta(n)>
*
* donde "ident" es la sustitucion identidad. La resolucion tendra exito si 
* existe alguna derivacion en la que <=G sea <=true. Diremos que theta(n) es la
* respuesta computada.
*
* 	Utilizaremos la siguiente regla de seleccion de las ecuaciones del
*    objetivo: elegir la ecuacion mas a la izquierda. No importa la ecuacion
*    seleccionada, el resultado sera el mismo, solo que desarrollaremos otras
*    posibilidades del arbol de resolucion del objetivo.
*
*	Sin embargo, para que el metodo funcione, la regla RU unicamente
* puede ser aplicada a ecuaciones no triviales. Por esta razon, en lugar
* de partir del estado inicial formado de todo el objetivo, se deberia partir
* de un estado en el que las ecuaciones son no triviales. Al desechar estas
* ecuaciones triviales en la unificacion inicial, posteriormente se necesita
* efectuar una reconciliacion con la ecuaciones triviales desechadas, para
* preservar los axiomas de igualdad.
*
* 	En cualquier caso, para lograr una mayor eficiencia y reducir
* el espacio de busqueda, creemos que este ultimo paso deberia 'realizarse
* de alguna manera' antes de la aplicacion reiterada de la regla RU. Con
* esta idea en mente se nos ocurre definir el metodo semantico como un
* sistema de transicion de estados compuesto por dos reglas:
*	RU1.- Aplicable unicamente a ecuaciones triviales para resumir
*	el proceso de reconciliacion de soluciones.
*	RU2.- Aplicable a ecuaciones no triviales del mismo modo que lo
*	hace la regla anterior RU.
*
* 	Nuestro sistema de transicion de estados para modelar el metodo de
* resolucion semantico guarda un indudable parecido con la definicion
* de narrowing basico implementado en este programa. Las dos reglas que
* lo describen son las siguientes.
*
* RU1:
*               g=Fsel(G) && Trivial(g) && sigma=mgu(g.theta)
*	 ---------------------------------------------------------
*               <<=G, theta> -RU-> <<=(G-{g}), theta.sigma>
*
*
* RU2:
*	 g=Fsel(G) && +Trivial(g) && e<<O(E) && sigma=mgu(g.theta, e)
* 	--------------------------------------------------------------
*		<<=G, theta> -RU-> <<=(G-{g}), theta.sigma>
*
* OBSERVACIONES:
*
* Una vez calculada O(E), los arboles de resolucion que obtengamos seran 
* finitos. En el peor de los casos el espacio de busqueda sera todo un arbol 
* N-ario de M niveles, siendo N, el numero de ecuaciones de la semantica 
* operacional O(E), y M el numero de ecuaciones del objetivo. No encontraremos 
* ramas infinitas, todas seran de EXITO o de FALLO.
*
*
**************************************************************************/


/* Aplicacion de la  regla RU1.
A RU1() se le suministrara el estado que se analiza en ese momento, y una
ecuacion del objetivo de ese estado (que no queda eliminada pero que no
debe ser tratada de nuevo posteriormente)
En caso de fracaso devolvera NULL. En caso de exito devolvera un puntero al
mismo estado que se le paso de parametro, pero habiendo modificado su
campo de sustitucion (que ahora llevara implicita la ecuacion recien tratada. */

PRIVATE struct stat *RU1(struct stat *estado, struct arbol *ecu)
{
	struct arbol *ecuaux=NULL, *izq=NULL, *der=NULL;
	struct subs *subaux=NULL, *mgu=NULL;

	st.BN_RU1_TECAt++;
	ecuaux=copiecu(ecu);
	aplicar(&ecuaux, estado->alfa);
	der=ecuaux->herm; ecuaux->herm=NULL; izq=ecuaux;
	if (unificador(izq, der, &mgu))
	     {	subaux = componer(estado->alfa, mgu);
		borrasub(estado->alfa); estado->alfa=subaux;
		st.BN_RU1_TECAe++;
	     }
	else estado=NULL;
	borraecu(izq); izq=NULL;
	borraecu(der); der=NULL;
	borrasub(mgu); mgu=NULL;
	return estado;
}


/* El procedimiento resolverRU1()  es invocado con una lista de un solo
estado en el que aparece una lista con todas las ecuaciones triviales. El
procedimiento invoca a RU1() con todas y cada una de estas ecuaciones. Al
final, si no se ha producido ningun fallo, devuelve una lista con un estado
de exito (aunque la lista de ecuaciones no es vacia, por tanto tendra que
ser eliminada en el procedimiento invocador) en cuya sustitucion aparecen
de forma implicita todas las ecuaciones triviales. */

PRIVATE struct stat *resolverRU1(struct stat *L)
{
  struct stat *S=NULL;
  struct lista *e=NULL;

  for (S=L; S; S=S->sig) /* Para cada estado de la lista de estados (normal-
			 mente la lista solo constara de un estado) */
	for (e=S->ecus;e;e=e->sig) /* para cada ecuacion de la lista */
	     if (RU1(S,e->ecu)==NULL) return NULL; /* fallo */
  return L;
}


/* Aplicacion de la  regla RU2.
A RU2() se le suministrara el estado que se analiza en ese momento, una
ecuacion del objetivo de ese estado (la que habria que eliminar en caso de
exito al encontrar una ecuacion de O(E) que se pudiese unificar), y una
ecuacion de O(E), estandarizada, con la que unificar.
En caso de fracaso devolvera NULL. En caso de exito devolvera un puntero a un 
estado de resolucion formado por: un puntero a una lista de ecuaciones en la 
que la ecuacion, "g", se ha eliminado; un puntero a una substitucion; y el 
puntero a siguiente inicializado a NULL (el campo "raiz" que no se utiliza 
para nada, se inicializara tambien a NULL).  */

PRIVATE struct stat *RU2(struct stat *estado, struct arbol *ecu_g,
				struct arbol *ecu_oe)
{
  struct stat *estaux=NULL;
  struct arbol *g=NULL;
  struct subs *subaux=NULL;

  st.BN_RU2t++;
  /* Hay que comprobar si la ecuacion estandarizada, ecu_oe unifica
  sintacticamente con la instancia de la ecuacion, g, resultado de
  aplicar la sustitucion estado->alfa. */
  g=copiecu(ecu_g);
  aplicar(&g,estado->alfa);
  subaux=NULL;
  if (unificador(g,ecu_oe,&subaux)) {
	st.BN_RU2e++;
	if ((estaux=(struct stat *)malloc(sizeof(struct stat)))==NULL)
				memerror();
	estaux->ecus=estado->ecus->sig; /* Esta simplificacion se debe a que 
		el objetivo que hay que eliminar siempre esta en cabeza. Para 
		implementar la regla en toda su generalidad habria que 
		implementar una funcion eliminar(estado->ecus, ecu_g) */
	estaux->raiz=NULL;
	estaux->alfa=componer(estado->alfa,subaux);
	estaux->sig=NULL;
  }
  borraecu(g);
  borrasub(subaux);
  return estaux;
}


/*************************************************************************/
/************************** CALCULO EN ANCHURA ************************/

/* El procedimiento resolverRU2()  generara el arbol de resolucion de objetivos,
todos los niveles, que obtiene uno a uno en un calculo en anchura. Tiene como
argumento un conjunto de estados de resolucion. Cuando se aplique resolverRU2()
a la resolucion de un objetivo, G::= <=e1,..,eN, la lista de partida estara
formada por un solo estado <G,  (e1,..,eN), Ident>. Devuelve una lista de
estados de exito, <<=true, raiz, theta>, que se derivan del conjunto de estados
inicial (raiz no se utiliza).
Por cada nivel de arbol, tendremos que realizar los siguientes pasos:

A) Aplicar RU2(), para cada estado del nivel, para la primera ecuacion de la
lista de ecuaciones del estado, para cada ecuacion estandarizada de O(E).
	** Los exitos de RU2() , G1= <<=true, theta(i)>:se enlazan en una
	   lista de estados, R, que se devolvera como resultado.
	** Los exitos de RU2() , G2= <<=Gi, theta(i)>:se enlazan en una
	   lista de estados, L2, que formara el siguiente nivel del arbol
	   de resolucion a analizar.
	** Si el estado devuelto por RU2() es un FALLO (NULL) pasar a
	   la siguiente iteracion.
B) El algoritmo terminara cuando la lista de estados, L, del nivel alcanzado 
sea vacia, es decir, el apuntador que la referencia sea NULL.

Si el proceso de resolucion no tiene exito en la unificacion del objetivo
con las ecuaciones de O(E), no encontramos ninguna rama de exito, La lista que
almacena el resultado estara vacia, R=NULL; Asi que un valor devuelto, NULL, 
manifestara ese fracaso.
Esta funcion dejara inalterado el estado que se le pasa como argumento. */

PRIVATE struct stat *resolverRU2(struct stat *L)
{
  struct arbol 	*g, /* puntero a la  ecuacion del "objetivo" de un
		       estado, seleccionada.*/
		*stdet; /* puntero a copia estandarizada de una ecuacion 
			de la teoria. */
  struct lista	*et, *uaux; /* variables auxiliares */
  struct stat 	*L1=NULL, /*puntero auxiliar a nivel de estados que se va 
			    a analizar. Recorrera la lista. */
		*L2=NULL, /* puntero a lista que almacena estados de tipo
			   G2. Siguiente nivel a analizar */
		*L2fin=NULL, /* puntero al final de la lista L2. */
		*G=NULL, /* puntero estados de exito de tipo G1 o de tipo G2. */
		*R=NULL, /* puntero lista que almacena estados de exito G1. 
			    Es la respuesta devuelta. */
		*Rfin=NULL; /* puntero al final de la lista Rfin. */
  struct stat *Lini; /* para chapuza */

  if (L == NULL) return (struct stat *) NULL; /* Para que seguir. */

  /* Esta linea es la chapuza necesaria para lograr una mayor eficiencia.
     Nuevamente tendremos que borrar la lista de estados de una forma especial,
     en este caso sin borrar las ecuaciones. */
  Lini = L;

  while(L!=NULL) { /* Mientras haya niveles que analizar */
    for (L1=L,L2= NULL; L1; L1=L1->sig) 
    { /* Mientras nivel de estados de no vacio,para cada estado de L1 */
	/* Para la 1ra ecuacion g de la lista de ecus del estado L1 */
	g=L1->ecus->ecu;
	for(et=oeini; et; et= et->sig)
	{ /* Para cada ecuacion de O(E) */
		stdet= estandar(et->ecu); /* estandariza ec. O(E).*/
		G= RU2(L1, g, stdet);
		if (G!=NULL) { /* Estado de exito tipo G1 o G2 */
		    if (G->ecus==NULL) {
			/* Es un estado de tipo G1 anyadirlo a lista de 
			estados de exito R. */
			if (R==NULL) R=G;
			else Rfin->sig=G;
			Rfin=G;
			if (trace) {
			  printf("\nRESOLVER : ESTADO DE EXITO G1:\n");
			  verestado(G);
			};
		   }
		   else { /* anyadir estado tipo G2 a lista de ests L2.*/
			if (L2==NULL) L2=G;
			else L2fin->sig=G;
			L2fin=G;
			if (trace) {
			  printf("\nRESOLVER : ESTADO DE EXITO G2:\n");
			  verestado(G);
			};
	  	  }
		};
	  	borraecu(stdet);
	};
    };
    if (L != Lini)  borrastatNE(L);
    L=L2; /* siguiente nivel a analizar */
    if (trace) {
    	printf("RESOLVER : lista de estados a resolver, siguiente nivel.\n");
	verestado(L);
    };
 };

  return R;
}

/******************* COMPOSICION PARALELA ******************************
***************************************************************************
En la version actual del programa esta operacion no es necesaria ya que
sus efectos son realizados previamente por la regla R1.
Reconcilia los estados de exito, obtenidos al aplicar resolver() a las
ecuaciones flat no triviales del objetivo, con las ecuaciones flat triviales.
Realiza la llamada composicion paralela, (||), de dos sustituciones, basandose
en el resultado teorico:

	(1)	sigma(||)theta = sigma.mgu(theta.sigma)

La respuesta computada, theta, viene dada en terminos de composicion paralela:

	(2)	theta = theta'(||)mgu(Gt)

donde theta' es el resultado obtenido mediante resolver(), y Gt son las 
ecuaciones flat triviales del objetivo. Teniendo en cuenta (1) el resultado (2)
quedaria como sigue:

	(3)	theta = theta'.mgu(mgu(Gt).theta')

Sabemos que Gt={x1=z1, x2=zk, ..., xk=zk, ...} , donde los xk, al ser variables 
frescas nunca coincidiran, puede que exista algun zk que coincida con otro zi, 
pero nunca habra un xk que coincida con otro xi. Por esta razon, mgu(Gt)=Gt y 
podemos escribir (3) de la forma:

	(4)	theta = theta'.mgu(Gt.theta')

que sera en ultimo termino el calculo que realiza la funcion conciliar().

*******************************************************************************/

/* Conciliar() juega el mismo papel que BN1() en el proceso de narrowing. 
Implementa la composicion paralela para obtener las respuestas computadas. 
Intuitivamente se corresponde con el proceso de realizar una reconciliacion de 
los resultados obtenidos con las ecuaciones triviales del objetivo, hasta ahora
no tenidas en cuenta, para que no se violen los axiomas de igualdad.

Implementaremos la composicion paralela en los siguientes terminos. Para cada 
sustitucion de un estado de exito, para cada ecuacion trivial gt->ecu, tendremos
que:
		Para todas las ecuaciones triviales {
			Si unificador(gt->ecu.s->alfa, mgu)
				entonces EXITO: mgu
				sino descartar s->alfa
		}

Devolvera las sustituciones, s->alfa.mgu, no descartadas en una lista de estados
R. Conciliar() no altera la lista de exitos que se le pasan como parametro. */

PRIVATE struct stat *conciliar(struct stat *exitos)
{
  struct stat *s, *R=NULL, *Rfin=NULL, *estaux;
  struct lista *gt;
  struct subs *mgu, *subaux;
  struct arbol *ecuaux, *izq, *der;
	
  for (s=exitos; s; s=s->sig)
  {/* para cada sustitucion de un estado de exito */
	mgu=NULL; subaux=NULL;
	for (gt=tgoalini; gt; gt=gt->sig)
	{/* para cada ecuacion gt->ecu, trivial. El proceso total conduce al
	    calculo del mgu(Gt.s->alfa), donde Gt son las ecuaciones planas 
	    triviales del objetivo*/
		ecuaux=copiecu(gt->ecu);
		aplicar(&ecuaux, s->alfa);
		aplicar(&ecuaux, mgu);
		der=ecuaux->herm; ecuaux->herm=NULL; izq=ecuaux;
		if (unificador(izq, der, &subaux)) {
			borraecu(izq); izq=NULL;
			borraecu(der); der=NULL;
			mgu = componer(mgu, subaux);
			borrasub(subaux); subaux=NULL;
		}
		else {
		    	borraecu(izq); izq=NULL;
			borraecu(der); der=NULL;
			borrasub(mgu);
			break;
		}
	}
	if (gt==NULL) { /* hubo exito en la unificacion */
	  /* crear estado de exito */
	  if ( (estaux= (struct stat *) malloc(sizeof(struct stat))) == NULL)
					memerror();
	  estaux->ecus= NULL;
	  estaux->raiz= NULL;
	  estaux->alfa= componer(s->alfa, mgu);
	  estaux->sig= NULL;
	  borrasub(mgu);
	  /* Unirlo a la lista resultado R */
	  if (R==NULL) R=estaux;
	  else Rfin->sig=estaux;
	  Rfin=estaux;
	}
  }
  return R;
}

/**************************************************************************
Resolucion de un objetivo mediante el Metodo Alpuente. Los pasos a seguir son
bien sencillos:

A)	Se genera la lista de ecuaciones flat del objetivo: fgoalini, fgoalfin.
B)	Se hace el SPLIT de la lista de ecuaciones falt: ntgoalini, tgoalini.
C)	Se genera un estado de resolucion inicial, utilizando la lista de 
	ecuaciones flat triviales.
D)	Se resuelve este estado aplicando R1 hasta obtener un estado de
	exito cuya sustitucion representa a las ecuaciones triviales.
E)	Se genera un nuevo estado de resolucion, utilizando la lista de
	ecuaciones flat no triviales, y la sustitucion obtenida en el paso
	anterior.
F)	Se aplica Resolucion por Unificacion contra O(E) a dicho estado
	(utilizando la regla R2), para obtener las respuestas computadas.
G)	Se visualizan las instancias del objetivo de acuerdo a las respuestas
	conciliadas.
**************************************************************************/

PUBLIC resob_sem()
{
	struct stat *S, /* puntero al estado inicial */
		*R; /* puntero a la lista de estados de exito. */
	struct subs *subaux=NULL;

	start_st(SEMANTICO);
	fgoalini=flat(goalini, &fgoalfin);
	if (trace) verlis("\nLista de ecuaciones FLAT:",fgoalini);
	split(fgoalini);
	if (tgoalini)
	   {	S = genstat(tgoalini);
		R=resolverRU1(S);
		if (R==NULL) return; /* Error al aplicar resolverR1 */
		subaux=copisub(R->alfa);
		borrastat(S);
		/*borrastat(R);*/ /* Este borrado ya esta hecho al
					borrar. S */
		borralis(tgoalini);
	   }
	S=genstat(ntgoalini);
	S->alfa=subaux;  /* En esta sustitucion del estado inicial
			   quedan contempladas las ecuaciones triviales
			   para ser tenidas en cuenta en el resto de
			   los calculos. */
	R = resolverRU2(S);
	verres(R);
	borrastat(S);
	/*borrastat(R); */   /* Este borrado ya esta hecho al
					borrar. S */

	borralis(ntgoalini);
	stop_st();
}


